@@ -3,9 +3,9 @@
     Copyright (C) 2002  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation
+    
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 // ODBC backend by Michel Stol (michel@powerdns.com)
@@ -366,9 +366,10 @@ void ODBCBackend::getUnfreshSlaveInfos( std::vector< DomainInfo > *pDomains )
       di.zone = m_diQuery.m_name;
 
     if ( m_diQuery.m_nullResult[ 2 ] == SQL_NULL_DATA )
-      di.master = "";
-    else
-      di.master = m_diQuery.m_master;
+      di.masters.clear();
+    else {
+      stringtok(di.masters, m_diQuery.m_master, " \t");
+    }
 
     if ( m_diQuery.m_nullResult[ 3 ] == SQL_NULL_DATA )
       di.last_check = 0;
