@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002-2005  PowerDNS.COM BV
+    Copyright (C) 2002-2007  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <errno.h>
@@ -24,16 +24,17 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fstream>
+#include <fcntl.h>
 #include <sstream>
-
+#include <boost/bind.hpp>
+#include <boost/algorithm/string.hpp>
 using namespace std;
 
 #include "dns.hh"
 #include "dnsbackend.hh"
 #include "bindbackend2.hh"
 #include "dnspacket.hh"
-
-#include "zoneparser.hh"
+#include "zoneparser-tng.hh"
 #include "bindparser.hh"
 #include "logger.hh"
 #include "arguments.hh"
@@ -49,30 +50,38 @@ using namespace std;
     on start of query, we find the best zone to answer from
 */
 
-Bind2Backend::name_id_map_t Bind2Backend::s_name_id_map;
-
 // this map contains BB2DomainInfo structs, each of which contains a *pointer* to domain data
-Bind2Backend::id_zone_map_t Bind2Backend::s_id_zone_map, Bind2Backend::s_staging_zone_map;
+shared_ptr<Bind2Backend::State> Bind2Backend::s_state;
 
-/* the model is that everything works from the s_id_zone_map, which is left alone most of the time.
-   
-   Any updates which might fail, build a new zone map in s_staging_zone_map
-   Then we swap atomically, and clear the copy.
+/* the model is that all our state hides in s_state. This State instance consists of the id_zone_map, which contains all our zone information, indexed by id.
+   Then there is the name_id_map, which allows us to map a query to a zone id.
 
-   Zone id's are kept constant so s_name_id_map doesn't need to be updated, except on removal or addition.
+   The s_state is never written to, and it is a reference counted shared_ptr. Any function which needs to access the state
+   should do so by making a shared_ptr copy of it, and do all its work on that copy.
+
+   When I said s_state is never written to, I lied. No elements are ever added to it, or removed from it.
+   Its values however may be changed, but not the keys. 
+
+   When it is necessary to change the State, a deep copy is made, which is changed. Afterwards, 
+   the s_state pointer is made to point to the new State.
+
+   Anybody who is currently accessing the original holds a reference counted handle (shared_ptr) to it, which means it will stay around
+   To save memory, we hold the records as a shared_ptr as well.
+
+   Changes made to s_state directly should take the s_state_lock, so as to prevent writing to a stale copy.
 */
 
 int Bind2Backend::s_first=1;
 
 pthread_mutex_t Bind2Backend::s_startup_lock=PTHREAD_MUTEX_INITIALIZER;
-pthread_mutex_t Bind2Backend::s_zonemap_lock=PTHREAD_MUTEX_INITIALIZER;
-
+pthread_mutex_t Bind2Backend::s_state_lock=PTHREAD_MUTEX_INITIALIZER;
+string Bind2Backend::s_binddirectory;  
 /* when a query comes in, we find the most appropriate zone and answer from that */
 
 BB2DomainInfo::BB2DomainInfo()
 {
   d_loaded=false;
-  d_last_check=0;
+  d_lastcheck=0;
   d_checknow=false;
   d_status="Unknown";
 }
@@ -87,7 +96,13 @@ bool BB2DomainInfo::current()
   if(d_checknow)
     return false;
 
-  if(!d_checkinterval || (time(0) - d_lastcheck < d_checkinterval ) || d_filename.empty())
+  if(!d_checkinterval) 
+    return true;
+
+  if(time(0) - d_lastcheck < d_checkinterval)
+    return true;
+  
+  if(d_filename.empty())
     return true;
 
   return (getCtime()==d_ctime);
@@ -113,17 +128,22 @@ void BB2DomainInfo::setCtime()
 
 void Bind2Backend::setNotified(uint32_t id, uint32_t serial)
 {
-  s_id_zone_map[id].d_lastnotified=serial;
+  Lock l(&s_state_lock);
+  s_state->id_zone_map[id].d_lastnotified=serial;
 }
 
 void Bind2Backend::setFresh(uint32_t domain_id)
 {
-  s_id_zone_map[domain_id].d_last_check=time(0);
+  Lock l(&s_state_lock);
+  s_state->id_zone_map[domain_id].d_lastcheck=time(0);
 }
 
 bool Bind2Backend::startTransaction(const string &qname, int id)
 {
-  BB2DomainInfo &bbd=s_id_zone_map[d_transaction_id=id];
+  shared_ptr<State> state = s_state; // is only read from
+
+  const BB2DomainInfo &bbd=state->id_zone_map[d_transaction_id=id];
+
   d_transaction_tmpname=bbd.d_filename+"."+itoa(random());
   d_of=new ofstream(d_transaction_tmpname.c_str());
   if(!*d_of) {
@@ -134,7 +154,7 @@ bool Bind2Backend::startTransaction(const string &qname, int id)
   }
   
   *d_of<<"; Written by PowerDNS, don't edit!"<<endl;
-  *d_of<<"; Zone '"+bbd.d_name+"' retrieved from master "<<bbd.d_master<<endl<<"; at "<<nowTime()<<endl;
+  *d_of<<"; Zone '"+bbd.d_name+"' retrieved from master "<<endl<<"; at "<<nowTime()<<endl; // insert master info here again
 
   return true;
 }
@@ -143,11 +163,13 @@ bool Bind2Backend::commitTransaction()
 {
   delete d_of;
   d_of=0;
-  if(rename(d_transaction_tmpname.c_str(),s_id_zone_map[d_transaction_id].d_filename.c_str())<0)
-    throw DBException("Unable to commit (rename to: '"+s_id_zone_map[d_transaction_id].d_filename+"') AXFRed zone: "+stringerror());
+  shared_ptr<State> state = s_state;
 
+  // this might fail if s_state was cycled during the AXFR
+  if(rename(d_transaction_tmpname.c_str(), state->id_zone_map[d_transaction_id].d_filename.c_str())<0)
+    throw DBException("Unable to commit (rename to: '" + state->id_zone_map[d_transaction_id].d_filename+"') AXFRed zone: "+stringerror());
 
-  queueReload(&s_id_zone_map[d_transaction_id]);
+  queueReload(&state->id_zone_map[d_transaction_id]);
 
   d_transaction_id=0;
 
@@ -169,7 +191,9 @@ bool Bind2Backend::abortTransaction()
 bool Bind2Backend::feedRecord(const DNSResourceRecord &r)
 {
   string qname=r.qname;
-  string domain=s_id_zone_map[d_transaction_id].d_name;
+
+  const shared_ptr<State> state = s_state;
+  string domain = state->id_zone_map[d_transaction_id].d_name;
 
   if(!stripDomainSuffix(&qname,domain)) 
     throw DBException("out-of-zone data '"+qname+"' during AXFR of zone '"+domain+"'");
@@ -178,17 +202,15 @@ bool Bind2Backend::feedRecord(const DNSResourceRecord &r)
 
   // SOA needs stripping too! XXX FIXME - also, this should not be here I think
   switch(r.qtype.getCode()) {
-  case QType::TXT:
-    *d_of<<qname<<"\t"<<r.ttl<<"\t"<<r.qtype.getName()<<"\t\""<<r.content<<"\""<<endl;
-    break;
   case QType::MX:
-    if(!stripDomainSuffix(&content,domain))
+    if(!stripDomainSuffix(&content, domain))
       content+=".";
+  case QType::SRV:
     *d_of<<qname<<"\t"<<r.ttl<<"\t"<<r.qtype.getName()<<"\t"<<r.priority<<"\t"<<content<<endl;
     break;
   case QType::CNAME:
   case QType::NS:
-    if(!stripDomainSuffix(&content,domain))
+    if(!stripDomainSuffix(&content, domain))
       content+=".";
     *d_of<<qname<<"\t"<<r.ttl<<"\t"<<r.qtype.getName()<<"\t"<<content<<endl;
     break;
@@ -203,19 +225,22 @@ bool Bind2Backend::feedRecord(const DNSResourceRecord &r)
 void Bind2Backend::getUpdatedMasters(vector<DomainInfo> *changedDomains)
 {
   SOAData soadata;
-  for(id_zone_map_t::iterator i=s_id_zone_map.begin();i!=s_id_zone_map.end();++i) {
-    if(!i->second.d_master.empty())
+
+  //  Lock l(&s_state_lock); // we don't really change the zone map, just flip a bit
+
+  for(id_zone_map_t::iterator i = s_state->id_zone_map.begin(); i != s_state->id_zone_map.end() ; ++i) {
+    if(!i->second.d_masters.empty())
       continue;
     soadata.serial=0;
     try {
-      getSOA(i->second.d_name,soadata); // we might not *have* a SOA yet
+      this->getSOA(i->second.d_name, soadata); // we might not *have* a SOA yet, but this might trigger a load of it
     }
     catch(...){}
     DomainInfo di;
     di.id=i->first;
     di.serial=soadata.serial;
     di.zone=i->second.d_name;
-    di.last_check=i->second.d_last_check;
+    di.last_check=i->second.d_lastcheck;
     di.backend=this;
     di.kind=DomainInfo::Master;
     if(!i->second.d_lastnotified)            // don't do notification storm on startup 
@@ -228,14 +253,15 @@ void Bind2Backend::getUpdatedMasters(vector<DomainInfo> *changedDomains)
 
 void Bind2Backend::getUnfreshSlaveInfos(vector<DomainInfo> *unfreshDomains)
 {
-  for(id_zone_map_t::const_iterator i=s_id_zone_map.begin();i!=s_id_zone_map.end();++i) {
-    if(i->second.d_master.empty())
+  shared_ptr<State> state = s_state;
+  for(id_zone_map_t::const_iterator i = state->id_zone_map.begin(); i != state->id_zone_map.end() ; ++i) {
+    if(i->second.d_masters.empty())
       continue;
     DomainInfo sd;
     sd.id=i->first;
     sd.zone=i->second.d_name;
-    sd.master=i->second.d_master;
-    sd.last_check=i->second.d_last_check;
+    sd.masters=i->second.d_masters;
+    sd.last_check=i->second.d_lastcheck;
     sd.backend=this;
     sd.kind=DomainInfo::Slave;
     SOAData soadata;
@@ -255,14 +281,15 @@ void Bind2Backend::getUnfreshSlaveInfos(vector<DomainInfo> *unfreshDomains)
 
 bool Bind2Backend::getDomainInfo(const string &domain, DomainInfo &di)
 {
-  for(id_zone_map_t::const_iterator i=s_id_zone_map.begin();i!=s_id_zone_map.end();++i) {
+  shared_ptr<State> state = s_state;
+  for(id_zone_map_t::const_iterator i = state->id_zone_map.begin(); i != state->id_zone_map.end() ; ++i) {
     if(i->second.d_name==domain) {
       di.id=i->first;
       di.zone=domain;
-      di.master=i->second.d_master;
-      di.last_check=i->second.d_last_check;
+      di.masters=i->second.d_masters;
+      di.last_check=i->second.d_lastcheck;
       di.backend=this;
-      di.kind=i->second.d_master.empty() ? DomainInfo::Master : DomainInfo::Slave;
+      di.kind=i->second.d_masters.empty() ? DomainInfo::Master : DomainInfo::Slave;
       di.serial=0;
       try {
 	SOAData sd;
@@ -296,24 +323,17 @@ static string canonic(string ret)
   return ret;
 }
 
-static Bind2Backend *us;
-
-static void InsertionCallback(unsigned int domain_id, const string &domain, const string &qtype, const string &content, int ttl, int prio)
-{
-  us->insert(domain_id, domain, qtype, content, ttl, prio);
-}
-
 set<string> contents;
 
-/** This function adds a record to a domain with a certain id. 
+/** THIS IS AN INTERNAL FUNCTION! It does moadnsparser prio impedence matching
+    This function adds a record to a domain with a certain id. 
     Much of the complication is due to the efforts to benefit from std::string reference counting copy on write semantics */
-void Bind2Backend::insert(int id, const string &qnameu, const string &qtype, const string &content, int ttl=300, int prio=25)
+void Bind2Backend::insert(shared_ptr<State> stage, int id, const string &qnameu, const QType &qtype, const string &content, int ttl=300, int prio=25)
 {
+  BB2DomainInfo bb2 = stage->id_zone_map[id];
   Bind2DNSRecord bdr;
 
-  BB2DomainInfo& bb2=s_staging_zone_map[id];
-
-  vector<Bind2DNSRecord>& records=*bb2.d_records;
+  vector<Bind2DNSRecord>& records=*bb2.d_records; 
 
   bdr.qname=toLower(canonic(qnameu));
   if(bdr.qname==toLower(bb2.d_name))
@@ -321,15 +341,28 @@ void Bind2Backend::insert(int id, const string &qnameu, const string &qtype, con
   else if(bdr.qname.length() > bb2.d_name.length())
     bdr.qname.resize(bdr.qname.length() - (bb2.d_name.length() + 1));
   else
-    throw AhuException("Trying to insert non-zone data, name='"+bdr.qname+"', zone='"+s_staging_zone_map[id].d_name+"'");
+    throw AhuException("Trying to insert non-zone data, name='"+bdr.qname+"', zone='" + s_state->id_zone_map[id].d_name+"'");
 
   bdr.qname.swap(bdr.qname);
 
   if(!records.empty() && bdr.qname==(records.end()-1)->qname)
     bdr.qname=(records.end()-1)->qname;
 
-  bdr.qtype=QType(qtype.c_str()).getCode();
-  bdr.content=canonic(content); // I think this is wrong, the zoneparser should not come up with . terminated stuff XXX FIXME
+  bdr.qtype=qtype.getCode();
+  bdr.content=content; 
+
+  if(bdr.qtype == QType::MX || bdr.qtype == QType::SRV) { 
+    prio=atoi(bdr.content.c_str());
+    
+    string::size_type pos = bdr.content.find_first_not_of("0123456789");
+    if(pos != string::npos)
+      erase_head(bdr.content, pos);
+    trim_left(bdr.content);
+  }
+  
+  if(bdr.qtype==QType::CNAME || bdr.qtype==QType::MX || bdr.qtype==QType::NS || bdr.qtype==QType::AFSDB)
+    bdr.content=canonic(bdr.content); // I think this is wrong, the zoneparser should not come up with . terminated stuff XXX FIXME
+
   set<string>::const_iterator i=contents.find(bdr.content);
   if(i!=contents.end())
    bdr.content=*i;
@@ -339,25 +372,27 @@ void Bind2Backend::insert(int id, const string &qnameu, const string &qtype, con
 
   bdr.ttl=ttl;
   bdr.priority=prio;
-
+  
   records.push_back(bdr);
 }
 
 void Bind2Backend::reload()
 {
-  for(id_zone_map_t::iterator i=us->s_id_zone_map.begin();i!=us->s_id_zone_map.end();++i) 
+  Lock l(&s_state_lock);
+  for(id_zone_map_t::iterator i = s_state->id_zone_map.begin(); i != s_state->id_zone_map.end(); ++i) 
     i->second.d_checknow=true;
 }
 
 string Bind2Backend::DLReloadNowHandler(const vector<string>&parts, Utility::pid_t ppid)
 {
+  shared_ptr<State> state = s_state;
   ostringstream ret;
 
   for(vector<string>::const_iterator i=parts.begin()+1;i<parts.end();++i) {
-    if(s_name_id_map.count(*i)) {
-      BB2DomainInfo& bbd=s_id_zone_map[s_name_id_map[*i]];
+    if(state->name_id_map.count(*i)) {
+      BB2DomainInfo& bbd=state->id_zone_map[state->name_id_map[*i]];
       
-      us->queueReload(&bbd);
+      queueReload(&bbd);
       ret<< *i << ": "<< (bbd.d_loaded ? "": "[rejected]") <<"\t"<<bbd.d_status<<"\n";      
     }
     else
@@ -372,10 +407,12 @@ string Bind2Backend::DLReloadNowHandler(const vector<string>&parts, Utility::pid
 string Bind2Backend::DLDomStatusHandler(const vector<string>&parts, Utility::pid_t ppid)
 {
   ostringstream ret;
+  shared_ptr<State> state = s_state;;
+      
   if(parts.size() > 1) {
     for(vector<string>::const_iterator i=parts.begin()+1;i<parts.end();++i) {
-      if(s_name_id_map.count(*i)) {
-	BB2DomainInfo& bbd=s_id_zone_map[s_name_id_map[*i]];
+      if(state->name_id_map.count(*i)) {
+	BB2DomainInfo& bbd=state->id_zone_map[state->name_id_map[*i]];  // XXX s_name_id_map needs trick as well
 	ret<< *i << ": "<< (bbd.d_loaded ? "": "[rejected]") <<"\t"<<bbd.d_status<<"\n";      
     }
       else
@@ -383,7 +420,7 @@ string Bind2Backend::DLDomStatusHandler(const vector<string>&parts, Utility::pid
     }    
   }
   else
-    for(id_zone_map_t::iterator i=us->s_id_zone_map.begin(); i!=us->s_id_zone_map.end(); ++i) 
+    for(id_zone_map_t::iterator i=state->id_zone_map.begin(); i!=state->id_zone_map.end(); ++i) 
       ret<< i->second.d_name << ": "<< (i->second.d_loaded ? "": "[rejected]") <<"\t"<<i->second.d_status<<"\n";      
 
   if(ret.str().empty())
@@ -395,15 +432,16 @@ string Bind2Backend::DLDomStatusHandler(const vector<string>&parts, Utility::pid
 
 string Bind2Backend::DLListRejectsHandler(const vector<string>&parts, Utility::pid_t ppid)
 {
+  shared_ptr<State> state = s_state;
+
   ostringstream ret;
-  for(id_zone_map_t::iterator j=us->s_id_zone_map.begin();j!=us->s_id_zone_map.end();++j) 
+  for(id_zone_map_t::iterator j = state->id_zone_map.begin(); j != state->id_zone_map.end(); ++j) 
     if(!j->second.d_loaded)
       ret<<j->second.d_name<<"\t"<<j->second.d_status<<endl;
 	
   return ret.str();
 }
 
-
 Bind2Backend::Bind2Backend(const string &suffix)
 {
 #if __GNUC__ >= 3
@@ -417,30 +455,49 @@ Bind2Backend::Bind2Backend(const string &suffix)
   if(!s_first) {
     return;
   }
-   
   s_first=0;
-  
+  s_state = shared_ptr<State>(new State);
   loadConfig();
 
   extern DynListener *dl;
-  us=this;
   dl->registerFunc("BIND-RELOAD-NOW", &DLReloadNowHandler);
   dl->registerFunc("BIND-DOMAIN-STATUS", &DLDomStatusHandler);
   dl->registerFunc("BIND-LIST-REJECTS", &DLListRejectsHandler);
 }
 
+Bind2Backend::~Bind2Backend()
+{
+
+}
+
 void Bind2Backend::rediscover(string *status)
 {
   loadConfig(status);
 }
 
+static void prefetchFile(const std::string& fname)
+{
+#if 0
+  static int fd;
+  if(fd > 0)
+    close(fd);
+  fd=open(fname.c_str(), O_RDONLY);
+  if(fd < 0)
+    return;
+
+  posix_fadvise(fd, 0, 0, POSIX_FADV_WILLNEED);
+#endif 
+}
+
 void Bind2Backend::loadConfig(string* status)
 {
   // Interference with createSlaveDomain()
-  Lock l(&s_zonemap_lock);
+  Lock l(&s_state_lock);
   
   static int domain_id=1;
-  s_staging_zone_map.clear();
+
+  shared_ptr<State> staging = shared_ptr<State>(new State);
+
   if(!getArg("config").empty()) {
     BindParser BP;
     try {
@@ -450,22 +507,30 @@ void Bind2Backend::loadConfig(string* status)
       L<<Logger::Error<<"Error parsing bind configuration: "<<ae.reason<<endl;
       throw;
     }
-
-    ZoneParser ZP;
       
     vector<BindDomainInfo> domains=BP.getDomains();
     
-    us=this;
-
-    d_binddirectory=BP.getDirectory();
-    ZP.setDirectory(d_binddirectory);
-    ZP.setCallback(&InsertionCallback);  
+    s_binddirectory=BP.getDirectory();
+    //    ZP.setDirectory(d_binddirectory);
 
     L<<Logger::Warning<<d_logprefix<<" Parsing "<<domains.size()<<" domain(s), will report when done"<<endl;
     
     int rejected=0;
     int newdomains=0;
 
+    //    random_shuffle(domains.begin(), domains.end());
+    struct stat st;
+      
+    for(vector<BindDomainInfo>::iterator i=domains.begin(); i!=domains.end(); ++i) 
+    {
+      if(stat(i->filename.c_str(), &st) == 0) {
+	i->d_dev = st.st_dev;
+	i->d_ino = st.st_ino;
+      }
+    }
+
+    sort(domains.begin(), domains.end()); // put stuff in inode order
+
     for(vector<BindDomainInfo>::const_iterator i=domains.begin();
 	i!=domains.end();
 	++i) 
@@ -477,11 +542,10 @@ void Bind2Backend::loadConfig(string* status)
 
 	BB2DomainInfo* bbd=0;
 
-	if(!s_name_id_map.count(i->name)) { // is it fully new?
-	  bbd=&s_staging_zone_map[domain_id];
+	if(!s_state->name_id_map.count(i->name)) { // is it fully new?
+	  bbd=&staging->id_zone_map[domain_id];
 	  bbd->d_id=domain_id++;
-	  s_name_id_map[i->name]=bbd->d_id;
-
+	
 	  // this isn't necessary, we do this on the actual load
 	  //	  bbd->d_records=shared_ptr<vector<Bind2DNSRecord> > (new vector<Bind2DNSRecord>);
 
@@ -490,34 +554,41 @@ void Bind2Backend::loadConfig(string* status)
 	  bbd->d_loaded=false;
 	}
 	else {  // no, we knew about it already
-	  s_staging_zone_map[s_name_id_map[i->name]]=s_id_zone_map[s_name_id_map[i->name]];
-	  bbd=&s_staging_zone_map[s_name_id_map[i->name]];
+	  staging->id_zone_map[s_state->name_id_map[i->name]] = s_state->id_zone_map[s_state->name_id_map[i->name]]; // these should all be read-only on s_state
+	  bbd = &staging->id_zone_map[s_state->name_id_map[i->name]];
 	}
 	
+	staging->name_id_map[i->name]=bbd->d_id; // fill out name -> id map
 
 	// overwrite what we knew about the domain
 	bbd->d_name=i->name;
 	bbd->d_filename=i->filename;
-	bbd->d_master=i->master;
+	bbd->d_masters=i->masters;
 	
 	if(!bbd->d_loaded || !bbd->current()) {
-	  L<<Logger::Info<<d_logprefix<<" parsing '"<<i->name<<"' from file '"<<i->filename<<"'"<<endl;
+	  //	  L<<Logger::Info<<d_logprefix<<" parsing '"<<i->name<<"' from file '"<<i->filename<<"'"<<endl;
 	  
 	  try {
-	    // we need to allocate a new vector so we don't kill the original
+	    // we need to allocate a new vector so we don't kill the original, which is still in use!
 	    bbd->d_records=shared_ptr<vector<Bind2DNSRecord> > (new vector<Bind2DNSRecord>); 
 
-	    ZP.parse(i->filename, i->name, bbd->d_id); // calls callback for us
-	    L<<Logger::Info<<d_logprefix<<" sorting '"<<i->name<<"'"<<endl;
+	    ZoneParserTNG zpt(i->filename, i->name, BP.getDirectory());
+	    DNSResourceRecord rr;
+	    while(zpt.get(rr)) {
+	      insert(staging, bbd->d_id, rr.qname, rr.qtype, rr.content, rr.ttl, rr.priority);
+	    }
 
-	    sort(s_staging_zone_map[bbd->d_id].d_records->begin(), s_staging_zone_map[bbd->d_id].d_records->end());
+	    //	    ZP.parse(i->filename, i->name, bbd->d_id); // calls callback for us
+	    //	    L<<Logger::Info<<d_logprefix<<" sorting '"<<i->name<<"'"<<endl;
 
-	    s_staging_zone_map[bbd->d_id].setCtime();
-	    s_staging_zone_map[bbd->d_id].d_loaded=true; 
-	    s_staging_zone_map[bbd->d_id].d_status="parsed into memory at "+nowTime();
+	    sort(staging->id_zone_map[bbd->d_id].d_records->begin(), staging->id_zone_map[bbd->d_id].d_records->end());
+	    
+	    staging->id_zone_map[bbd->d_id].setCtime();
+	    staging->id_zone_map[bbd->d_id].d_loaded=true; 
+	    staging->id_zone_map[bbd->d_id].d_status="parsed into memory at "+nowTime();
 
 	    contents.clear();
-	    //  s_staging_zone_map[bbd->d_id].d_records->swap(*s_staging_zone_map[bbd->d_id].d_records);
+	    //  s_stage->id_zone_map[bbd->d_id].d_records->swap(*s_staging_zone_map[bbd->d_id].d_records);
 	  }
 	  catch(AhuException &ae) {
 	    ostringstream msg;
@@ -525,7 +596,17 @@ void Bind2Backend::loadConfig(string* status)
 
 	    if(status)
 	      *status+=msg.str();
-	    s_staging_zone_map[bbd->d_id].d_status=msg.str();
+	    staging->id_zone_map[bbd->d_id].d_status=msg.str();
+	    L<<Logger::Warning<<d_logprefix<<msg.str()<<endl;
+	    rejected++;
+	  }
+	  catch(exception &ae) {
+	    ostringstream msg;
+	    msg<<" error at "+nowTime()+" parsing '"<<i->name<<"' from file '"<<i->filename<<"': "<<ae.what();
+
+	    if(status)
+	      *status+=msg.str();
+	    staging->id_zone_map[bbd->d_id].d_status=msg.str();
 	    L<<Logger::Warning<<d_logprefix<<msg.str()<<endl;
 	    rejected++;
 	  }
@@ -539,25 +620,26 @@ void Bind2Backend::loadConfig(string* status)
     // figure out which domains were new and which vanished
     int remdomains=0;
     set<string> oldnames, newnames;
-    for(id_zone_map_t::const_iterator j=s_id_zone_map.begin();j!=s_id_zone_map.end();++j) {
+    for(id_zone_map_t::const_iterator j=s_state->id_zone_map.begin();j != s_state->id_zone_map.end();++j) {
       oldnames.insert(j->second.d_name);
     }
-    for(id_zone_map_t::const_iterator j=s_staging_zone_map.begin();j!=s_staging_zone_map.end();++j) {
+    for(id_zone_map_t::const_iterator j=staging->id_zone_map.begin(); j!= staging->id_zone_map.end(); ++j) {
       newnames.insert(j->second.d_name);
     }
 
     vector<string> diff;
     set_difference(oldnames.begin(), oldnames.end(), newnames.begin(), newnames.end(), back_inserter(diff));
     remdomains=diff.size();
-        
+
+#if 0        
     // remove domains from the *name* map, delete their pointer
     for(vector<string>::const_iterator k=diff.begin();k!=diff.end(); ++k) {
       L<<Logger::Error<<"Removing domain: "<<*k<<endl;
-      s_name_id_map.erase(*k);
+      s_state->name_id_map.erase(*k);
     }
 
-    // now remove from the s_id_zone_map
-    for(id_zone_map_t::iterator j=s_id_zone_map.begin();j!=s_id_zone_map.end();++j) { // O(N*M)
+    // now remove from the s_state->id_zone_map
+    for(id_zone_map_t::iterator j=s_state->id_zone_map.begin();j!=s_state->id_zone_map.end();++j) { // O(N*M)
       for(vector<string>::const_iterator k=diff.begin();k!=diff.end();++k)
 	if(j->second.d_name==*k) {
 	  L<<Logger::Error<<"Removing records from zone '"<<j->second.d_name<<"' from memory"<<endl;
@@ -568,14 +650,14 @@ void Bind2Backend::loadConfig(string* status)
 	  break;
 	}
     }
+#endif
 
     // count number of entirely new domains
     vector<string> diff2;
     set_difference(newnames.begin(), newnames.end(), oldnames.begin(), oldnames.end(), back_inserter(diff2));
     newdomains=diff2.size();
-
-    s_id_zone_map.swap(s_staging_zone_map); // commit
-    s_staging_zone_map.clear();             // and cleanup
+    
+    s_state.swap(staging); // and boy do we hope this is a threadsafe operation!
 
     // report
     ostringstream msg;
@@ -591,38 +673,35 @@ void Bind2Backend::loadConfig(string* status)
 void Bind2Backend::nukeZoneRecords(BB2DomainInfo *bbd)
 {
   bbd->d_loaded=0; // block further access
-  bbd->d_records->clear(); // empty the vector of Bind2DNSRecords
+  bbd->d_records = shared_ptr<vector<Bind2DNSRecord> > (new vector<Bind2DNSRecord>);
 }
 
 
 void Bind2Backend::queueReload(BB2DomainInfo *bbd)
 {
-  Lock l(&s_zonemap_lock);
+  Lock l(&s_state_lock);
 
-  s_staging_zone_map.clear(); 
+  shared_ptr<State> staging(new State);
 
   // we reload *now* for the time being
 
   try {
-    nukeZoneRecords(bbd);
-    
-    ZoneParser ZP;
-    us=this;
+    nukeZoneRecords(bbd); // ? do we need this?
+    staging->id_zone_map[bbd->d_id]=s_state->id_zone_map[bbd->d_id];
+    staging->id_zone_map[bbd->d_id].d_records=shared_ptr<vector<Bind2DNSRecord> > (new vector<Bind2DNSRecord>);  // nuke it
 
-    ZP.setDirectory(d_binddirectory);
-    ZP.setCallback(&InsertionCallback);  
-
-    s_staging_zone_map[bbd->d_id]=s_id_zone_map[bbd->d_id];
-    s_staging_zone_map[bbd->d_id].d_records=shared_ptr<vector<Bind2DNSRecord> > (new vector<Bind2DNSRecord>);  // nuke it
-
-    ZP.parse(bbd->d_filename, bbd->d_name, bbd->d_id);
-    
-    sort(s_staging_zone_map[bbd->d_id].d_records->begin(), s_staging_zone_map[bbd->d_id].d_records->end());
-    s_staging_zone_map[bbd->d_id].setCtime();
+    ZoneParserTNG zpt(bbd->d_filename, bbd->d_name, s_binddirectory);
+    DNSResourceRecord rr;
+    while(zpt.get(rr)) {
+      insert(staging, bbd->d_id, rr.qname, rr.qtype, rr.content, rr.ttl, rr.priority);
+    }
+        
+    sort(staging->id_zone_map[bbd->d_id].d_records->begin(), staging->id_zone_map[bbd->d_id].d_records->end());
+    staging->id_zone_map[bbd->d_id].setCtime();
     
     contents.clear();
 
-    s_id_zone_map[bbd->d_id]=s_staging_zone_map[bbd->d_id]; // move over
+    s_state->id_zone_map[bbd->d_id]=staging->id_zone_map[bbd->d_id]; // move over
 
     bbd->setCtime();
     // and raise d_loaded again!
@@ -636,6 +715,11 @@ void Bind2Backend::queueReload(BB2DomainInfo *bbd)
     msg<<" error at "+nowTime()+" parsing '"<<bbd->d_name<<"' from file '"<<bbd->d_filename<<"': "<<ae.reason;
     bbd->d_status=msg.str();
   }
+  catch(exception &ae) {
+    ostringstream msg;
+    msg<<" error at "+nowTime()+" parsing '"<<bbd->d_name<<"' from file '"<<bbd->d_filename<<"': "<<ae.what();
+    bbd->d_status=msg.str();
+  }
 }
 
 bool operator<(const Bind2DNSRecord &a, const string &b)
@@ -655,15 +739,17 @@ void Bind2Backend::lookup(const QType &qtype, const string &qname, DNSPacket *pk
 
   string domain=toLower(qname);
 
-  bool mustlog=arg().mustDo("query-logging");
+  bool mustlog=::arg().mustDo("query-logging");
   if(mustlog) 
     L<<Logger::Warning<<"Lookup for '"<<qtype.getName()<<"' of '"<<domain<<"'"<<endl;
 
-  while(!s_name_id_map.count(domain) && chopOff(domain));
+  shared_ptr<State> state = s_state;
 
-  name_id_map_t::const_iterator iditer=s_name_id_map.find(domain);
+  while(!state->name_id_map.count(domain) && chopOff(domain));
 
-  if(iditer==s_name_id_map.end()) {
+  name_id_map_t::const_iterator iditer=state->name_id_map.find(domain);
+
+  if(iditer==state->name_id_map.end()) {
     if(mustlog)
       L<<Logger::Warning<<"Found no authoritative zone for "<<qname<<endl;
     d_handle.d_list=false;
@@ -681,27 +767,25 @@ void Bind2Backend::lookup(const QType &qtype, const string &qname, DNSPacket *pk
   if(strcasecmp(qname.c_str(),domain.c_str()))
     d_handle.qname=qname.substr(0,qname.size()-domain.length()-1); // strip domain name
 
-  d_handle.parent=this;
   d_handle.qtype=qtype;
   d_handle.domain=qname.substr(qname.size()-domain.length());
 
-  d_handle.d_records=s_id_zone_map[iditer->second].d_records; // give it a copy
-  if(!d_handle.d_records->empty()) {
-    BB2DomainInfo& bbd=s_id_zone_map[iditer->second];
-    if(!bbd.d_loaded) {
-      d_handle.reset();
-      throw DBException("Zone temporarily not available (file missing, or master dead)"); // fsck
-    }
+  BB2DomainInfo& bbd = state->id_zone_map[iditer->second];
+  if(!bbd.d_loaded) {
+    d_handle.reset();
+    throw DBException("Zone for '"+bbd.d_name+"' in '"+bbd.d_filename+"' temporarily not available (file missing, or master dead)"); // fsck
+  }
     
-    if(!bbd.current()) {
-      L<<Logger::Warning<<"Zone '"<<bbd.d_name<<"' ("<<bbd.d_filename<<") needs reloading"<<endl;
-      queueReload(&bbd);
-      d_handle.d_records=s_id_zone_map[iditer->second].d_records; // give it a *fresh* copy
-    }
+  if(!bbd.current()) {
+    L<<Logger::Warning<<"Zone '"<<bbd.d_name<<"' ("<<bbd.d_filename<<") needs reloading"<<endl;
+    queueReload(&bbd);  // how can this be safe - ok, everybody should have their own reference counted copy of 'records'
+    state = s_state;
   }
-  else {
+
+  d_handle.d_records = state->id_zone_map[iditer->second].d_records; // give it a reference counted copy
+  
+  if(d_handle.d_records->empty())
     DLOG(L<<"Query with no results"<<endl);
-  }
 
   pair<vector<Bind2DNSRecord>::const_iterator, vector<Bind2DNSRecord>::const_iterator> range;
 
@@ -717,6 +801,8 @@ void Bind2Backend::lookup(const QType &qtype, const string &qname, DNSPacket *pk
   else {
     d_handle.d_iter=range.first;
     d_handle.d_end_iter=range.second;
+    d_handle.mustlog = mustlog;
+
   }
 
   d_handle.d_list=false;
@@ -724,8 +810,7 @@ void Bind2Backend::lookup(const QType &qtype, const string &qname, DNSPacket *pk
 
 Bind2Backend::handle::handle()
 {
-  //  d_records=0;
-  count=0;
+  mustlog=false;
 }
 
 bool Bind2Backend::get(DNSResourceRecord &r)
@@ -736,13 +821,13 @@ bool Bind2Backend::get(DNSResourceRecord &r)
   if(!d_handle.get(r)) {
     d_handle.reset();
 
-    if(arg().mustDo("query-logging"))
+    if(::arg().mustDo("query-logging"))
       L<<"End of answers"<<endl;
 
     return false;
   }
-  if(arg().mustDo("query-logging"))
-    L<<"Returning: '"<<r.qtype.getName()<<"' of '"<<r.qname<<"', content: '"<<r.content<<"'"<<endl;
+  if(d_handle.mustlog)
+    L<<Logger::Warning<<"Returning: '"<<r.qtype.getName()<<"' of '"<<r.qname<<"', content: '"<<r.content<<"', prio: "<<r.priority<<endl;
   return true;
 }
 
@@ -786,18 +871,17 @@ bool Bind2Backend::handle::get_normal(DNSResourceRecord &r)
 
 bool Bind2Backend::list(const string &target, int id)
 {
-  if(!s_id_zone_map.count(id))
+  shared_ptr<State> state = s_state;
+  if(!state->id_zone_map.count(id))
     return false;
 
   d_handle.reset(); 
   DLOG(L<<"Bind2Backend constructing handle for list of "<<id<<endl);
 
-  d_handle.d_qname_iter=s_id_zone_map[id].d_records->begin();
-  d_handle.d_qname_end=s_id_zone_map[id].d_records->end();   // iter now points to a vector of pointers to vector<BBResourceRecords>
+  d_handle.d_records=state->id_zone_map[id].d_records; // give it a copy, which will stay around
+  d_handle.d_qname_iter= d_handle.d_records->begin();
+  d_handle.d_qname_end=d_handle.d_records->end();   // iter now points to a vector of pointers to vector<BBResourceRecords>
 
-  d_handle.d_records=s_id_zone_map[id].d_records; // give it a copy --- WHY??? XXX FIXME
-
-  d_handle.parent=this;
   d_handle.id=id;
   d_handle.d_list=true;
   return true;
@@ -822,9 +906,13 @@ bool Bind2Backend::handle::get_list(DNSResourceRecord &r)
 
 bool Bind2Backend::isMaster(const string &name, const string &ip)
 {
-  for(id_zone_map_t::iterator j=us->s_id_zone_map.begin();j!=us->s_id_zone_map.end();++j) 
-    if(j->second.d_name==name)
-      return j->second.d_master==ip;
+  for(id_zone_map_t::iterator j=s_state->id_zone_map.begin();j!=s_state->id_zone_map.end();++j) {
+    if(j->second.d_name==name) {
+      for(vector<string>::const_iterator iter = j->second.d_masters.begin(); iter != j->second.d_masters.end(); ++iter)
+	if(*iter==ip)
+	  return true;
+    }
+  }
   return false;
 }
 
@@ -867,7 +955,7 @@ bool Bind2Backend::superMasterBackend(const string &ip, const string &domain, co
 bool Bind2Backend::createSlaveDomain(const string &ip, const string &domain, const string &account)
 {
   // Interference with loadConfig(), use locking
-  Lock l(&s_zonemap_lock);
+  Lock l(&s_state_lock);
 
   string filename = getArg("supermaster-destdir")+'/'+domain;
   
@@ -893,20 +981,20 @@ bool Bind2Backend::createSlaveDomain(const string &ip, const string &domain, con
   int newid=0;
   // Find a free zone id nr.  
   
-  if (!s_id_zone_map.empty()) {
-    id_zone_map_t::reverse_iterator i = s_id_zone_map.rbegin();
+  if (!s_state->id_zone_map.empty()) {
+    id_zone_map_t::reverse_iterator i = s_state->id_zone_map.rbegin();
     newid = i->second.d_id + 1;
   }
   
-  BB2DomainInfo &bbd = s_id_zone_map[newid];
+  BB2DomainInfo &bbd = s_state->id_zone_map[newid];
 
   bbd.d_records = shared_ptr<vector<Bind2DNSRecord> >(new vector<Bind2DNSRecord>);
   bbd.d_name = domain;
   bbd.setCheckInterval(getArgAsNum("check-interval"));
-  bbd.d_master = ip;
+  bbd.d_masters.push_back(ip);
   bbd.d_filename = filename;
 
-  s_name_id_map[domain] = bbd.d_id;
+  s_state->name_id_map[domain] = bbd.d_id;
   
   return true;
 }
@@ -919,11 +1007,11 @@ class Bind2Factory : public BackendFactory
       void declareArguments(const string &suffix="")
       {
          declare(suffix,"config","Location of named.conf","");
-         declare(suffix,"example-zones","Install example zones","no");
+	 //         declare(suffix,"example-zones","Install example zones","no");
          declare(suffix,"check-interval","Interval for zonefile changes","0");
          declare(suffix,"supermaster-config","Location of (part of) named.conf where pdns can write zone-statements to","");
          declare(suffix,"supermasters","List of IP-addresses of supermasters","");
-         declare(suffix,"supermaster-destdir","Destination directory for newly added slave zones",arg()["config-dir"]);
+         declare(suffix,"supermaster-destdir","Destination directory for newly added slave zones",::arg()["config-dir"]);
       }
 
       DNSBackend *make(const string &suffix="")
