@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2005  PowerDNS.COM BV
+    Copyright (C) 2005 - 2007 PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "rcpgenerator.hh"
@@ -57,7 +57,7 @@ void RecordTextReader::xfrTime(uint32_t &val)
   tm.tm_year-=1900;
   tm.tm_min-=1;
   
-  val=mktime(&tm);
+  val=(uint32_t)mktime(&tm);
 }
 
 void RecordTextReader::xfrIP(uint32_t &val)
@@ -65,12 +65,41 @@ void RecordTextReader::xfrIP(uint32_t &val)
   skipSpaces();
 
   if(!isdigit(d_string.at(d_pos)))
-    throw RecordTextException("expected digits at position "+lexical_cast<string>(d_pos)+" in '"+d_string+"'");
+    throw RecordTextException("while parsing IP address, expected digits at position "+lexical_cast<string>(d_pos)+" in '"+d_string+"'");
+
+  uint32_t octet=0;
+  val=0;
+  char count=0;
   
-  string ip;
-  xfrLabel(ip);
-  if(!IpToU32(ip, &val))
-    throw RecordTextException("unable to parse IP address '"+ip+"'");
+  for(;;) {
+    if(d_string.at(d_pos)=='.') {
+      val<<=8;
+      val+=octet;
+      octet=0;
+      count++;
+      if(count > 3)
+	break;
+    }
+    else if(isdigit(d_string.at(d_pos))) {
+      octet*=10;
+      octet+=d_string.at(d_pos) - '0';
+      if(octet > 255)
+	throw RecordTextException("unable to parse IP address");
+    }
+    else if(dns_isspace(d_string.at(d_pos))) 
+      break;
+    else
+      throw RecordTextException("unable to parse IP address, strange character: "+d_string.at(d_pos));
+
+    d_pos++;
+    if(d_pos == d_string.length())
+      break;
+  }
+  if(count<=3) {
+    val<<=8;
+    val+=octet;
+  }
+  val=ntohl(val);
 }
 
 
@@ -97,22 +126,21 @@ void RecordTextReader::xfr8BitInt(uint8_t &val)
     throw RecordTextException("Overflow reading 8 bit integer from record content"); // fixme improve
 }
 
-
-void RecordTextReader::xfrLabel(string& val, bool)
+void RecordTextReader::xfrLabel(string& val, bool) 
 {
   skipSpaces();
-
   val.clear();
   val.reserve(d_end - d_pos);
 
+  const char* strptr=d_string.c_str();
   while(d_pos < d_end) {
-    if(dns_isspace(d_string[d_pos]))
+    if(dns_isspace(strptr[d_pos]))
       break;
 
-    if(d_string[d_pos]=='\\' && d_pos < d_end - 1) 
+    if(strptr[d_pos]=='\\' && d_pos < d_end - 1 && strptr[d_pos+1]!='.')  // leave the \. escape around
       d_pos++;
 
-    val.append(1, d_string[d_pos]);      
+    val.append(1, strptr[d_pos]);      
     d_pos++;
   }
 
@@ -131,8 +159,9 @@ void RecordTextReader::xfrLabel(string& val, bool)
 void RecordTextReader::xfrBlob(string& val)
 {
   skipSpaces();
-  int pos=d_pos;
-  while(d_pos < d_end && !dns_isspace(d_string[d_pos]))
+  int pos=(int)d_pos;
+  const char* strptr=d_string.c_str();
+  while(d_pos < d_end && !dns_isspace(strptr[d_pos]))
     d_pos++;
 
   string tmp;
@@ -141,30 +170,75 @@ void RecordTextReader::xfrBlob(string& val)
   B64Decode(tmp, val);
 }
 
-void RecordTextReader::xfrText(string& val)
+
+static inline uint8_t hextodec(uint8_t val)
+{
+  if(val >= '0' && val<='9')
+    return val-'0';
+  else if(val >= 'A' && val<='F')
+    return 10+(val-'A');
+  else if(val >= 'a' && val<='f')
+    return 10+(val-'a');
+  else
+    throw RecordTextException("Unknown hexadecimal character '"+lexical_cast<string>(val)+"'");
+}
+
+
+void HEXDecode(const char* begin, const char* end, string& val)
+{
+  if((end - begin)%2)
+    throw RecordTextException("Hexadecimal blob with odd number of characters");
+
+  int limit=(int)(end-begin)/2;
+  val.resize(limit);
+  for(int n=0; n < limit; ++n) {
+    val[n] = hextodec(begin[2*n])*16 + hextodec(begin[2*n+1]); 
+  }
+}
+
+void RecordTextReader::xfrHexBlob(string& val)
 {
   skipSpaces();
-  if(d_string[d_pos]!='"')
-    throw RecordTextException("Data field in DNS should start with quote (\") at position "+lexical_cast<string>(d_pos)+" of '"+d_string+"'");
+  int pos=(int)d_pos;
+  while(d_pos < d_end && !dns_isspace(d_string[d_pos]))
+    d_pos++;
 
+  HEXDecode(d_string.c_str()+pos, d_string.c_str() + d_pos, val);
+}
+
+void RecordTextReader::xfrText(string& val, bool multi)
+{
   val.clear();
   val.reserve(d_end - d_pos);
-  
-  while(++d_pos < d_end && d_string[d_pos]!='"') {
-    if(d_string[d_pos]=='\\' && d_pos+1!=d_end) {
-      ++d_pos;
+
+  while(d_pos != d_end) {
+    if(!val.empty())
+      val.append(1, ' ');
+
+    skipSpaces();
+    if(d_string[d_pos]!='"')
+      throw RecordTextException("Data field in DNS should start with quote (\") at position "+lexical_cast<string>(d_pos)+" of '"+d_string+"'");
+
+    val.append(1, '"');
+    while(++d_pos < d_end && d_string[d_pos]!='"') {
+      if(d_string[d_pos]=='\\' && d_pos+1!=d_end) {
+	val.append(1, d_string[d_pos++]);
+      }
+      val.append(1, d_string[d_pos]);
     }
-    val.append(1, d_string[d_pos]);
+    val.append(1,'"');
+    if(d_pos == d_end)
+      throw RecordTextException("Data field in DNS should end on a quote (\") in '"+d_string+"'");
+    d_pos++;
+    if(!multi)
+      break;
   }
-  if(d_pos == d_end)
-    throw RecordTextException("Data field in DNS should end on a quote (\") in '"+d_string+"'");
-  d_pos++;
 }
 
 void RecordTextReader::xfrType(uint16_t& val)
 {
   skipSpaces();
-  int pos=d_pos;
+  int pos=(int)d_pos;
   while(d_pos < d_end && !dns_isspace(d_string[d_pos]))
     d_pos++;
 
@@ -177,9 +251,9 @@ void RecordTextReader::xfrType(uint16_t& val)
 
 void RecordTextReader::skipSpaces()
 {
-  while(d_pos < d_end && dns_isspace(d_string[d_pos]))
+  const char* strptr = d_string.c_str();
+  while(d_pos < d_end && dns_isspace(strptr[d_pos]))
     d_pos++;
-
   if(d_pos == d_end)
     throw RecordTextException("missing field at the end of record content '"+d_string+"'");
 }
@@ -213,13 +287,30 @@ void RecordTextWriter::xfrIP(const uint32_t& val)
     d_string.append(1,' ');
 
   char tmp[17];
-  snprintf(tmp, sizeof(tmp)-1, "%u.%u.%u.%u", 
-	   (val >> 24)&0xff,
-	   (val >> 16)&0xff,
-	   (val >>  8)&0xff,
-	   (val      )&0xff);
-  
-  d_string+=tmp;
+  uint32_t ip=htonl(val);
+  uint8_t vals[4];
+
+  memcpy(&vals[0], &ip, sizeof(ip));
+
+  char *pos=tmp;
+
+  for(int n=0; n < 4; ++n) {
+    if(vals[n]<10) {
+      *(pos++)=vals[n]+'0';
+    } else if(vals[n] < 100) {
+      *(pos++)=(vals[n]/10) +'0';
+      *(pos++)=(vals[n]%10) +'0';
+    } else {
+      *(pos++)=(vals[n]/100) +'0';
+      vals[n]%=100;
+      *(pos++)=(vals[n]/10) +'0';
+      *(pos++)=(vals[n]%10) +'0';
+    }
+    if(n!=3)
+      *(pos++)='.';
+  }
+  *pos=0;
+  d_string.append(tmp, pos);
 }
 
 
@@ -230,7 +321,15 @@ void RecordTextWriter::xfrTime(const uint32_t& val)
   
   struct tm tm;
   time_t time=val; // Y2038 bug!
+#ifndef WIN32
   gmtime_r(&time, &tm);
+#else
+  struct tm* tmptr;
+  tmptr=gmtime(&time);
+  if(!tmptr)
+    throw RecordTextException("Unable to convert timestamp into pretty printable time");
+  tm=*tmptr;
+#endif
   
   char tmp[16];
   snprintf(tmp,sizeof(tmp)-1, "%04d%02d%02d" "%02d%02d%02d", 
@@ -268,7 +367,7 @@ void RecordTextWriter::xfrLabel(const string& val, bool)
       else
 	d_string.append(1,val[pos]);
   }
-  d_string.append(1,'.');
+  //  d_string.append(1,'.');
 }
 
 void RecordTextWriter::xfrBlob(const string& val)
@@ -279,25 +378,25 @@ void RecordTextWriter::xfrBlob(const string& val)
   d_string+=Base64Encode(val);
 }
 
-void RecordTextWriter::xfrText(const string& val)
+void RecordTextWriter::xfrHexBlob(const string& val)
 {
   if(!d_string.empty())
     d_string.append(1,' ');
-  d_string.append(1,'"');
 
-  if(val.find_first_of("\\\"") == string::npos)
-    d_string+=val;
-  else {
-    string::size_type end=val.size();
-    
-    for(string::size_type pos=0; pos < end; ++pos) {
-      if(val[pos]=='\'' || val[pos]=='"')
-	d_string.append(1,'\\');
-      d_string.append(1, val[pos]);
-    }
+  string::size_type limit=val.size();
+  char tmp[5];
+  for(string::size_type n = 0; n < limit; ++n) {
+    snprintf(tmp, sizeof(tmp)-1, "%02x", (unsigned char)val[n]);
+    d_string+=tmp;
   }
+}
 
-  d_string.append(1,'"');
+void RecordTextWriter::xfrText(const string& val, bool multi)
+{
+  if(!d_string.empty())
+    d_string.append(1,' ');
+
+  d_string.append(val);
 }
 
 
